#include <FK/FK.h>
#include <Fl/x.h>
#include "FKUT/Input.h"
#include "FKUT/Frame.h"
#include "FKUT/Fullscreen.h"


class CRobo {
private:
	fk_Block	block;
	fk_Model	blockModel;
	fk_Circle	circ;
	fk_Model	shadowModel;

	fk_Vector	vel, pos;
	fk_Vector	axis1, axis2;
	int			boostTime;
	fk_Vector	boostVec;

public:
	fk_Model	main;

	CRobo(void)
	{
		block.setSize(50.0, 70.0, 40.0);
		blockModel.setShape(&block);
		blockModel.setMaterial(Yellow);
		blockModel.glMoveTo(0.0, 35.0, 0.0);
		blockModel.setParent(&main, true);

		circ.setRadius(40.0);
		shadowModel.setShape(&circ);
		shadowModel.setMaterial(MatBlack);
		shadowModel.glVec(0.0, -1.0, 0.0);
		shadowModel.glMoveTo(0.0, 0.1, 0.0);

		boostTime = 0;

		return;
	};

	void entryScene(fk_Scene *argScene)
	{
		argScene->entryModel(&blockModel);
		argScene->entryModel(&shadowModel);
	};

	void update(fkut_InputInfo *joy)
	{
		axis1.set(double(joy->x[0])/1000.0, 0.0, double(joy->y[0])/1000.0);
		if(axis1.dist2() > 1.0) axis1.normalize();
		axis2.set(double(joy->x[1])/1000.0, 0.0, double(joy->y[1])/1000.0);
		if(axis2.dist2() > 1.0) axis2.normalize();

		if(boostTime > 0) {
			boostTime--;
			boostVec *= 0.3;
		} else if(joy->button[4] || joy->button[5]) {
			boostTime = 60;
			if(axis1.isZero() && axis2.isZero()) {
				boostVec = main.getVec()*2.0;
			} else {
				boostVec = main.getMatrix()*(axis1 + axis2);
			}
		} else {
			boostVec.init();
		}

		vel.y -= 1.0;

		if(axis1.x < -0.5 && axis2.x > 0.5) {
			vel.y += 1.5;
			if(vel.y > 20.0) vel.y = 20.0;
		} else if(axis1.x > 0.5 && axis2.x < -0.5) {
			vel.x *= 0.5;	vel.z *= 0.5;
			vel.y -= 3.0;
		} else if(axis1.z < -0.5 && axis2.z > 0.5) {
			main.glRotateWithVec(main.getPosition(), fk_Y, -0.02);
		} else if(axis1.z > 0.5 && axis2.z < -0.5) {
			main.glRotateWithVec(main.getPosition(), fk_Y, 0.02);
		}

		main.loTranslate(axis1*5.0 + axis2*5.0);
		main.glTranslate(vel - boostVec);

		// Փ˔
		pos = main.getPosition();
		if(pos.y < 0.0) {
			pos.y = 0.0;
			main.glMoveTo(pos);
			vel.y = 0.0;
		} else if(pos.y > 500.0) {
			pos.y = 500.0;
			main.glMoveTo(pos);
			vel.y = 0.0;
		}

		shadowModel.glMoveTo(pos.x, 0.2, pos.z);
	};
};

int main(int argc, char *argv[])
{
	Fl_Window	mainWindow(800, 600, "FK TEST");
	fk_Model	camera, blockModel, lightModel, floorModel;
	fk_Light	light;
	fk_Block	floor(1000.0, 1.0, 1000.0);
	fk_Scene	scene;
	fk_Window	window(0, 0, 800, 600);

	fkut_FPS	fps;

	mainWindow.end();
	fk_InitMaterial();

	CRobo		robo;


	// Ɩ̐ݒ
	lightModel.setShape(&light);
	lightModel.setMaterial(White);
	lightModel.glMoveTo(0.0, 0.0, 0.0);
	lightModel.glFocus(-1.0, -1.0, -1.0);

	// ̂̐ݒ
	floorModel.setShape(&floor);
	floorModel.setMaterial(Brown);
	floorModel.glMoveTo(0.0, -0.5, 0.0);

	// effBXvCXgɓo^
	scene.entryCamera(&camera);
	scene.entryModel(&floorModel);
	scene.entryModel(&lightModel);
	robo.entryScene(&scene);

	// EBhEփfBXvCXgo^
	window.setScene(&scene);

	// _̈ʒuƎpݒ
	camera.glMoveTo(0.0, 500.0, 1000.0);
	camera.glFocus(0.0, 0.0, 0.0);
	camera.glUpvec(0.0, 1.0, 0.0);
	camera.setParent(&robo.main, true);

	mainWindow.show();
	window.show();

	fkut_Input		input;
	fkut_InputInfo	joy;

	input.init(fl_display, fl_xid(&mainWindow));
	input.setAxisAssign(1, DIK_W, DIK_S, DIK_A, DIK_D);
	input.setKeyAssign(0, DIK_Z);
	input.setKeyAssign(1, DIK_X);
	input.setKeyAssign(2, DIK_C);
	input.setKeyAssign(3, DIK_V);

	CFullscreenControler	full;
	full.init(&mainWindow);
	//full.changeToFullscreen();

	while(true) {

		if(mainWindow.visible() == 0) {
			if(Fl::wait() == 0) {
				break;
			} else {
				continue;
			}
		}

		if(fps.update()) {
			if(window.drawWindow() == 0) break;
			if(Fl::check() == 0) break;
			if(window.winOpenStatus() == false) continue;
		}

		input.update();
		joy = input.getInputStatus();
		cout << "DIRECTION:";
		if(joy.up) cout << "U";
		if(joy.down) cout << "D";
		if(joy.left) cout << "L";
		if(joy.right) cout << "R";
		cout << " POV:" << joy.pov;
		cout << " AXIS1:" << joy.x[0] << "," << joy.y[0];
		cout << " AXIS2:" << joy.x[1] << "," << joy.y[1];
		cout << " BUTTON:";
		for(int i = 0; i < joy.BUTTON_NUM; i++) {
			if(joy.button[i]) cout << i;
		}
		cout << endl;

		robo.update(&joy);
	}

	//full.changeToWindow();

	return 0;
}
